#include "pch.h"
#include "GameProcessor.h"

using namespace TicTacToe;

using namespace std;
using namespace Windows::Foundation::Collections;

enum class gameResult
{
    Knot,
    Cross,
    Tie
};

bool checkEndOfGame(const vector<wchar_t>& gameBoard, wchar_t empty, wchar_t knot, wchar_t cross, gameResult& result, std::array<uint32_t, 3>& winningPath)
{
    /*
    |0|1|2|
    |3|4|5|
    |6|7|8|
    */
    const std::array<std::array<uint32_t, 3>, 8> permutations = 
    { {
        // Horizontals
        { {0, 1, 2} },
        { {3, 4, 5} },
        { {6, 7, 8} },

        // Verticals
        { {0, 3, 6} },
        { {1, 4, 7} },
        { {2, 5, 8} },

        // Diagonals
        { {0, 4, 8} },
        { {2, 4, 6} },
    } };

    for (auto permutation : permutations)
    {
        if (count_if(begin(permutation), end(permutation), [gameBoard, knot](uint32_t index)
        {
            return gameBoard[index] == knot;

        }) == permutation.size())
        {
            result = gameResult::Knot;
            winningPath = permutation;
            return true;
        }

        if (count_if(begin(permutation), end(permutation), [gameBoard, cross](uint32_t index)
        {
            return gameBoard[index] == cross;

        }) == permutation.size())
        {
            result = gameResult::Cross;
            winningPath = permutation;
            return true;
        }
    }

    if (count(begin(gameBoard), end(gameBoard), empty) == 0)
    {
        result = gameResult::Tie;
        return true;
    }
    return false;
}

GameResult GameProcessor::CheckEndOfGame(IVector<wchar_t>^ gameBoard, wchar_t empty, wchar_t knot, wchar_t cross, IVector<uint32>^ winningPath)
{
    gameResult result;
    std::array<uint32_t, 3> path;
    if (checkEndOfGame(to_vector(gameBoard), empty, knot, cross, result, path))
    {
        switch (result)
        {
        case gameResult::Knot:
            winningPath->Clear();
            winningPath->Append(path[0]);
            winningPath->Append(path[1]);
            winningPath->Append(path[2]);
            return GameResult::Knot;
        case gameResult::Cross:
            winningPath->Clear();
            winningPath->Append(path[0]);
            winningPath->Append(path[1]);
            winningPath->Append(path[2]);
            return GameResult::Cross;
        case gameResult::Tie:
            return GameResult::Tied;
        }
    }
    return GameResult::Active;
}