#pragma once
#include "BindableBase.h"

namespace TicTacToeLibrary
{
    ref class Cell;
    ref class GameProcessor;

    delegate void CellSelectedHandler(Cell^ sender);

    [Windows::UI::Xaml::Data::Bindable]
    [Windows::Foundation::Metadata::WebHostHidden]
    public ref class Cell sealed : public Common::BindableBase
    {
    public:
        Cell(uint32_t index, Windows::UI::Xaml::Media::Brush^ foregroundBrush, GameProcessor^ gameProcessor);

        property Platform::String^ Text
        {
            Platform::String^ get()
            {
                return ref new Platform::String(std::wstring(1, m_mark).c_str());
            }
        }

        property Windows::UI::Xaml::Input::ICommand^ SelectCommand
        {
            Windows::UI::Xaml::Input::ICommand^ get()
            {
                return m_selectCommand;
            }
        }

        property Windows::UI::Xaml::Media::Brush^ ForegroundBrush
        {
            Windows::UI::Xaml::Media::Brush^ get()
            {
                return m_foregroundBrush;
            }
            void set(Windows::UI::Xaml::Media::Brush^ value)
            {
                if (value != m_foregroundBrush)
                {
                    m_foregroundBrush = value;
                    OnPropertyChanged("ForegroundBrush");
                }
            }
        }

    internal:
        event CellSelectedHandler^ CellSelected;

        property uint32_t Index
        {
            uint32_t get() { return m_index; }
        }

        property wchar_t Mark
        {
            wchar_t get() { return m_mark; }
        }

        void Select(Platform::Object^ parameter);

    private:
        Platform::WeakReference m_gameProcessor;

        uint32_t m_index;
        wchar_t m_mark;
        Windows::UI::Xaml::Input::ICommand^ m_selectCommand;
        Windows::UI::Xaml::Media::Brush^ m_foregroundBrush;
    };
}

