#pragma once
#include "BindableBase.h"
#include "Cell.h"
#include "DelegateCommand.h"
#include "IPlayer.h"

namespace TicTacToeLibrary
{
    [Windows::UI::Xaml::Data::Bindable]
    [Windows::Foundation::Metadata::WebHostHidden]
    public ref class GameProcessor sealed : public Common::BindableBase
    {
    public:
        GameProcessor();

        property Windows::Foundation::Collections::IObservableVector<Cell^>^ Cells
        {
            Windows::Foundation::Collections::IObservableVector<Cell^>^ get() { return m_cells; }
        }

        property Windows::UI::Xaml::Input::ICommand^ NewGameCommand
        {
            Windows::UI::Xaml::Input::ICommand^ get() { return m_newGameCommand; }
        }

        property Windows::Foundation::Collections::IObservableVector<Platform::String^>^ PlayerOptions
        {
            Windows::Foundation::Collections::IObservableVector<Platform::String^>^ get() { return m_playerOptions; }
        }

        property int32 SelectedPlayerOption1;
        property int32 SelectedPlayerOption2;

        property bool CanCreateNewGame
        {
            bool get() { return !m_gameInProgress; }
        }

        property uint32 CrossWins
        {
            uint32 get() { return m_crossWins; }
        }

        property wchar_t CrossSymbol
        {
            wchar_t get() { return L'\u274C'; }
        }

        property uint32 NoughtWins
        {
            uint32 get() { return m_noughtWins; }
        }

        property wchar_t NoughtSymbol
        {
            wchar_t get() { return L'\u2B55'; }
        }

        property double BackgroundProgress
        {
            double get() { return m_backgroundProgress; }
        }

        property Platform::String^ GameResult
        {
            Platform::String^ get() { return m_gameResult; }
        }

    internal:
        property IPlayer^ CurrentPlayer
        {
            IPlayer^ get() { return m_currentPlayer; }
        }

        property bool IsThinking
        {
            bool get() { return m_currentThinkOp != nullptr; }
        }

    private:
        void CellSelected(Cell^ sender);
        void CheckEndOfGame();
        void CreateNewGame(Platform::Object^ parameter);
        bool CanSelectNewGame(Platform::Object^ parameter) { (void)parameter; return CanCreateNewGame; }
        void SwitchPlayers();
        void ProcessTurn(int32 index);

        static const wchar_t EmptySymbol = L' ';

        // The players.
        IPlayer^ m_player1;
        IPlayer^ m_player2;
        IPlayer^ m_currentPlayer;

        // The game board.
        Platform::Collections::Vector<Cell^>^ m_cells;
        std::array<wchar_t, 9> m_gameBoard;

        // Command object to create new game.
        Windows::UI::Xaml::Input::ICommand^ m_newGameCommand;

        // Flags an active game.
        bool m_gameInProgress;

        // Counts X and O wins.
        uint32 m_crossWins;
        uint32 m_noughtWins;

        // Computer's turn.
        Windows::Foundation::IAsyncOperationWithProgress<uint32, double>^ m_currentThinkOp;
        // % progress of computer's turn.
        double m_backgroundProgress;

        // Text that's displayed to let the user choose whether each player is human or computer-controlled.
        Platform::Collections::Vector<Platform::String^>^ m_playerOptions;

        // Used to define cells' foreground colors.
        // This is for simplicity, but a real app might add a property to the Cell class (maybe IsOnWinningPath) and 
        // use a XAML converter to define the brushes.
        Windows::UI::Xaml::Media::SolidColorBrush^ m_defaultButtonForegroundBrush;
        Windows::UI::Xaml::Media::SolidColorBrush^ m_winningPathButtonForegroundBrush;

        // Text for last game's result.
        Platform::String^ m_gameResult;
    };
}